import { 
  select, 
  tsv,
  csv,
  text,
} from 'd3';
import {
  forceSimulation,
  forceLink,
  forceManyBody,
  forceCenter,
} from 'd3-force';

import {optimal_leaf_order,} from 'reorder';

const radius = 3.5;
var last_select = [];

const renderDiagram = (links, nodes, max) => {
  
  const svg = d3.select('#svg1');
	const width = +svg.attr('width');
  const height = +svg.attr('height');
  const margin = { top: 60, right: 40, bottom: 50, left: 10};
  const innerHeight = height - margin.top -margin.bottom;
  const innerWidth = width - margin.left - margin.right;
  
  const g = svg.append('g')//.attr('transform', `translate(${margin.left}, ${margin.top})`);
  svg.attr('viewBox', [-100,600,600, 600])
  
  const drag = () => {
  	
    const dragStarted = (event) =>{
    	if(!event.active) simulation.alphaTarget(0.5).restart();
      event.subject.fx = event.subject.x;
      event.subject.fy = event.subject.y;
    };
    
    const dragged = (event) => {
    	event.subject.fx = event.x;
      event.subject.fy = event.y;
    };
    
    const dragEnded = (event) =>{
    	if(!event.active) simulation.alphaTarget(0);
      event.subject.fx = null;
      event.subject.fy = null;
    };
    
    return d3.drag()
      .on('start', dragStarted)
      .on('drag', dragged)
      .on('end', dragEnded)
  
  }
  
  const simulation = forceSimulation(nodes)
  	.force('link', d3.forceLink(links).id(d => d.id))
  	.force('charge', d3.forceManyBody())
  	.force('center', d3.forceCenter(width/2, height/2))

  
  const link = g.selectAll('line')
  	.data(links)
  	.enter()
  	.append('line')
  	.attr("stroke-width", 1)
		.attr("stroke","#C0C0BB")
  	.attr('opacity', 0.8)
  	.attr('id', d =>  'l' + d.source.id + '-' + d.target.id)
  
    //link.append('title')
  	//	.text(d => d.id)

  const node = g.selectAll('circle')
  	.data(nodes)
  	.enter()
  	.append('circle')
  	.attr('r', d => d.group*radius)
  	.attr('stroke-width', 1.5)
  	.attr('stroke', '#fff')
  	.attr('fill',  d => d3.schemeCategory10[d.group])
  	.attr('opacity', 0.7)
  	.attr('id',d => 'n' + d.id)
  	.call(drag())
  	.on('click', datum => {
      var id = datum.path[0].childNodes[0].__data__.id;
      d3.selectAll('rect')
      	.style('stroke-width', 1)
      	.style('stroke', '#C0C0BB');
      
      d3.select(last_select[0]).style('stroke', '#C0C0BB').style('stroke-width', 1);
      d3.select(last_select[1]).style('stroke', '#C0C0BB').style('stroke-width', 1);
      
      
      for(var i=1;i<=max;i++){
      	d3.select('#d' + i + '-' + id)
          .style('stroke-width', 3)
          .style('stroke', 'red');
        d3.select('#d' + id + '-' + i)
          .style('stroke-width', 3)
          .style('stroke', 'red');
      }
      d3.select('#text2')
        .text('Selected node:' + id + ', row and column has been highlighted.');
    });
  
  node.append('title')
  		.text(d => d.id)
  
  //zoom in, zoom out
  let transform;
  const zoomed = d3.zoom()
  	.extent([[0,0], [450,600],])
  	.scaleExtent([0.6,6])
  	.on("zoom", e => {
    	link.attr('transform', (transform = e.transform));
    	node.attr('transform', (transform = e.transform));
  	});
  d3.select('svg').call(zoomed)

  //handle position of nodes and links everytime
  const ticked = () => {
  	link.attr('x1', d => d.source.x)
    		.attr('y1', d => d.source.y)
    		.attr('x2', d => d.target.x)
    		.attr('y2', d => d.target.y)
    
    node.attr('cx', d => d.x)
    		.attr('cy', d => d.y)
  }
  
  simulation.on('tick', ticked)

};

const svgSize = 3200;
const gridSize = 7.5;

const renderAdjMatrix = (matrix, nodes) =>{
	
  const svg = d3.select('#ad').append('svg');
  const margin = { top: 20, right: 40, bottom: 50, left:50};
  
  svg.attr('id', 'svg2')
  	.attr('width', svgSize)
  	.attr('height', svgSize)
  
  const g = svg.append('g').attr('transform', `translate(${margin.left}, ${margin.top})`);
  g.append('text')
    .attr('id', 't1')
    .attr('y', +40)
    .attr('x', +15)
  	.attr('id', 'text2')
    .text("Click node in Force Directed Graph or click black grid in adjacency matrix to highlight");
  
  matrix.forEach((r,i) => {
    
  	var rectG = g.append('g').attr('transform', 'translate(10, 60)')
    	.attr('id', 'r'+ String(i))
    

    var max, min;
    var rect = rectG.selectAll('rect')
    	.data(r).enter()
    	.append('rect')
    	.attr('width', gridSize)
    	.attr('height', gridSize)
    	.attr('fill', '#8E6C8A')
    	.attr('x', d => d.x*gridSize)
    	.attr('y', d => d.y*gridSize)
    	.attr('fill-opacity', d => d.connect*1)
    	.attr('stroke', "#C0C0BB")
    	.attr('stroke-width', 0.8)
    	.attr('id', d => 'd'+ (d.x+1) + '-' + (d.y+1))
    	.on('click', datum => {
        
      	var id = datum.path[0].id;
        d3.selectAll('rect').style('stroke', '#C0C0BB').style('stroke-width', 1);
        d3.select('#'+ id).style('stroke', 'red').style('stroke-width', 2.5);
        
        //cancel select in node link diagram
        d3.select(last_select[0]).style('stroke', '#C0C0BB').style('stroke-width', 1);
        d3.select(last_select[1]).style('stroke', '#C0C0BB').style('stroke-width', 1);
        //last_select = [];
        
        //having links between two nodes
        if((datum.path[0].style['fillOpacity'] != '0')){
          var tmp = id.split('d');
          var tar = tmp[1].split('-');
          
          d3.select('#l'+ tar[0] + '-' + tar[1])
            .style('stroke-width', '3')
            .style('stroke', 'red');
          d3.select('#l' + tar[1] + '-' + tar[0])
            .style('stroke-width', '3')
            .style('stroke', 'red');
          
          last_select = ['#l'+ tar[0] + '-' + tar[1],'#l' + tar[1] + '-' + tar[0]];
          console.log(last_select)
          min = (parseInt(tar[0]) < parseInt(tar[1])) ? 0 : 1;
          max = (parseInt(tar[0]) >= parseInt(tar[1])) ? 0 : 1;
          d3.select('#text2')
            .text(`Edge between Node ${tar[min]} and Node ${tar[max]} has been selected`);

        }
        else{
          min = (tar[0] < tar[1]) ? 0 : 1;
          max = (tar[0] >= tar[1]) ? 0 : 1;
          d3.select('#text2')
            .text(`There's no edge between Node ${tar[min]} and Node ${tar[max]}`);
        }
      });
    	
  });
  
  
  //axis
  g.append("g")
		.attr("transform","translate(10,56.5)")
		.selectAll('text')
		.data(nodes)
		.enter()
		.append('text')
		.attr('x', (d,i) => i * gridSize + gridSize/2)
		.text(d => d.id)
		.style("text-anchor","middle")
		.style('font-size','4.5px')

	g.append('g')
    .attr('transform', 'translate(0,62)')
		.selectAll('text')
		.data(nodes)
		.enter()
		.append('text')
		.attr('y',(d,i) => i * gridSize + gridSize/2)
		.text(d => d.id)
		.style('text-anchor','middle')
		.style('font-size','4.5px')
  
	
};

 d3$1.csv('https://raw.githubusercontent.com/imsohandsome123/Data-Visualization/main/hw3%20Node-Link%20Diagrams%20vs.%20Adjacency%20Matrix/data.tsv').then((data) => {
		
  	const links = []
    const linksCnt = []
  
  	for(var i=0; i<1000; i++){
    	linksCnt.push(0);
    }
  
    var matrix = [];
  	for(var i=0; i<1000; i++){
    	var row = [];
      for(var j=0; j<1000; j++){
      	row.push({
        	x : j,
          y : i, 
          connect : 0
        });     
      }
      matrix.push(row);
    }
  
  	const key = Object.keys(data[0])[0]
    const d0 = key.split(' ');
    links.push({source: d0[0], target: d0[1], value:1});
  
		//count links of first data
    linksCnt[parseInt(d0[0])]++;
  	linksCnt[parseInt(d0[1])]++;
      
    //change the connected relation of the matrix of first data
    matrix[parseInt(d0[0])-1][parseInt(d0[1])-1].connect = 1;
    matrix[parseInt(d0[1])-1][parseInt(d0[0])-1].connect = 1;  
  
  
  	data.forEach(d => {
      
    	var tmp = d[key].split(' ');
      
      //new key
      d.source = tmp[0];
      d.target = tmp[1];
      d.value = 1;
      //delete old key
      delete d[key];     
      links.push(d);
      
      //count links of each nodes
      linksCnt[parseInt(d.source)]++;
      linksCnt[parseInt(d.target)]++;
      
      //change the connected relation of the matrix
      matrix[parseInt(d.source)-1][parseInt(d.target)-1].connect = 1;
      matrix[parseInt(d.target)-1][parseInt(d.source)-1].connect = 1;

    });
  	
  	const sourceList = links.map(d => parseInt(d.source)); 
  	const targetList = links.map(d => parseInt(d.target));
  	const max = Math.max( Math.max.apply(null, sourceList), Math.max.apply(null, targetList));
  	const min = Math.min( Math.min.apply(null, sourceList), Math.min.apply(null, targetList));
  
  	const nodes = [];
  	for(var i=min; i<max+1; i++){
    		nodes[i-1] = {
        	id : String(i),
          group : Math.ceil(linksCnt[i]/10)
        };
    };
  	
  	const connectedMatrix = [];
  	for(var i=0; i<max; i++){
    	var connectedRow = matrix[i].slice(0,max);
      connectedMatrix.push(connectedRow);
    }
  	const M = optimal_leaf_order()(connectedMatrix) 
    console.log(M)
  	
  	renderDiagram(links, nodes, max);
  	renderAdjMatrix(connectedMatrix, nodes);

});